/*F File_name: 
/*F	buckets.c
/*
/* Bucketize the output from a cell delay    */
/* stream and prepare for histogram plot.    */

/* Usage: cc -o buckets buckets.c                   */
/*        buckets num end infile outfile            */
/* where:                                           */
/*        num is the number of buckets desired      */
/*        start is the value for the low bucket     */
/*        end is the value for the high bucket      */

/* Modified by Ram Balakrishnan, March 1999.                  */
//
//N Note: 
//N     This program is altered to be compatiable with cascade.tcl program.
//N     Amalgamate this program with cascade.tcl using "xmktclapp".  

#include <stdio.h>
#include "synTraff.h"

#define MAX_BUCKETS 1000

/* #define VERBOSE 1 */

int ET_COMMAND_mkHis(ET_TCLARGS)
  {
    FILE *infile, *outfile;
    int i, index, counter, cumcount;
    int counthist[MAX_BUCKETS], total, numbuckets;
    float obs, num, startvalue = 0, endvalue, stepsize;

    if( argc < 5 )
      {
	Et_ResultF(interp, "Usage: mkHis num end infile outfile\n");
	return TCL_ERROR;
      }

    sscanf(argv[1],"%d",&numbuckets);
    if( (numbuckets > MAX_BUCKETS) || (numbuckets < 1) )
      {
	Et_ResultF(interp, "Entry for number of intervals is improper\n");
	return TCL_ERROR;
      }

    sscanf(argv[2],"%f",&endvalue);

    if( startvalue > endvalue )
      {
	Et_ResultF(interp, "Entry for end value is improper\n");
	return TCL_ERROR;
      }

   if ((infile = fopen(argv[3],"r")) == NULL)
   {
      Et_ResultF(interp, "Unable to open input file %s.\n",argv[3]);
      return TCL_ERROR;
   }

   if ((outfile = fopen(argv[4],"w")) == NULL)
   {
      Et_ResultF(interp, "Unable to open output file %s.\n",argv[4]);
      return TCL_ERROR;
   }

    stepsize = (endvalue - startvalue) / numbuckets;

    for( i = 0; i < numbuckets; i++ )
      counthist[i] = 0;

    total = 0;
    counter = 0;
    while( (i = fscanf(infile,"%f\n", &obs)) == 1 )
      {
	counter += 1;

	if( obs > endvalue )
	  {
	    counthist[numbuckets-1]++;
	    total++;
	    continue;
	  }

	if( obs < startvalue )
	  {
	    counthist[0]++;
	    total++;
	    continue;
	  }

	/* figure out bucket number for this observation */
	num = startvalue;
	index = 0;
	while( num < obs )
	  {
	    num += stepsize;
	    if( num < obs )
	      index++;
	  }
	counthist[index]++;
	total++;
      }

    /* Summary info */
    cumcount = 0;
    for( i = 0; i < numbuckets; i++ )
      {
	int count;
	count = counthist[i];
	cumcount += count;
/*	fprintf(outfile,"%4d   %6.3f   %6d   %8.5f  %8.5f   %8.5f\n",
	       i, startvalue + i * stepsize,
	       count, 
	       (float) 100.0 * count/total,
	       (float) 1.0 * cumcount/total,
	       (float) 1.0 - 1.0 * cumcount/total);
*/
	fprintf(outfile,"%6.3f   %8.5f\n", startvalue + i * stepsize, 
		        (float) 100.0 * count/total);
      }

   fclose(infile);
   fclose(outfile);

   return TCL_OK;
  }









