/*F File_name: 
/*F	fancyshift.c
/*
/*P Purpose:
/*P 	The purpose of this program is to take a zero mean
/*P 	synthetically generated long-range dependent time series
/*P     and do a mathematical transformation of all data points x
/*P     by computing b^((1-r)/(r+e))x. This should preserve
/*P     the self-similarity, but convert to a new range of numbers
/*P     (i.e., changing the spread from the smallest to the largest)
/*P     The mean of the time series will change.
/*P     The variance will also change, as well as the peak to mean ratio.
/*
/*U Usage:
/*U     cc -o fancyshift fancyshift.c -lm
/*U     fancyshift density < input_file > output_file
/*U        where the density value r is in the range (0,1). 
/*
/*E Examples:
/*E     fancyshift 2.0 0.45 0.1 < H5.original.dat > H5.fancyshift2.45.1.dat
/*E       (this changes range [-1,1] into range [0.5,2.0]).
/*E     fancyshift 2.0 0.85 0.1 < H5.original.dat > H5.fancyshift2.85.1.dat
/*E       (this changes range [-1,1] into range [0.896,1.116]).
/*E     fancyshift 2.0 0.15 0.1 < H5.original.dat > H5.fancyshift2.15.1.dat
/*E       (this changes range [-1,1] into range [0.094,10.556]).
/*E     fancyshift 2.0 0.45 0.01 < H5.original.dat > H5.fancyshift2.45.01.dat
/*E       (this changes range [-1,1] into range [0.436,2.29]).
/*
/*      General observations: larger density values produce
/*      much lower variance in the output stream; small density
/*      values result in more variability, slightly higher means,
/*      and a larger peak to mean ratio.
/*
/*A Author:
/*A  	Carey Williamson  (and modified by Ram Balakrishnan, February 1999)
/*
/*D Date:
/*D 	July 1996
/*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

main(argc, argv)
int argc;
char **argv;
{
  int i;
  float base = 2.0, density, epsilon = 0.1, num;

  if(argc < 2)
  {
    printf("Usage: fancyshift density < input_file > output_file\n");
    exit(-1);
  }

  density = atof(argv[1]);
  if( (density < 0.0) || (density > 1.0) )
    {
      printf("Sorry, but the density value needs to be in the range [0,1]\n");
      exit(-1);
    }

  while( (i = scanf("%f\n", &num)) == 1 )
    printf("%f\n", pow(base, num * (1.0 - density) / (density + epsilon)));

  return (0);
}
