/*F File_name: 
/*F	mult.c
/*
/*P Purpose:
/*P 	The purpose of this program is to take an input time series
/*P     at one time granularity and refine it into a finer-grained
/*P     time series with the same mean bit rate. The name implies
/*P     "multiplicating" the desired amount of traffic amongst the
/*P     subintervals, using multifractal wavelets. The user specifies
/*P     the number of subintervals to be created for each observation.
/*P     Uses a "" technique to generate subinterval values.
/*
/*N Note: 
/*N     This program is altered to be compatiable with cascade.tcl program.
/*N     Amalgamate this program with cascade.tcl using "xmktclapp".  
/*N     The pathname specified in compilation is specific for skorpio machine.
/*
/*U Usage:
/*U     gcc -o cascade MeanStd.c mult.c cascade.c -L/sw/arch/lib -ltk8.0 -ltcl8.0 -lX11 -lm -ldl
/*
/*E Example:
/*E     refine 10 0.5 0.5 H8.dat H8.mul.dat
/*
/*A Author:
/*A  	Ram Balakrishnan
/*
/*D Date:
/*D 	February 1999
/*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>    /* use man 3 log for the math functions */
#include "synTraff.h"

#define RND_MAX 0x7fffffff
#define MAX_INTERVALS 1000

/* The following two values, mean and standard deviation */
/* for normal distribution, can be used to influence the */
/* short-range correlations between the values generated */
/* in the subintervals, if desired.                      */


double ceil(double x);
double erand48(unsigned short []); 
double Normal(float, float); 

/* array used for initializing seeds for Normal distribution */
unsigned short Nvar1_seed[3] = {4,7,2};
unsigned short Nvar2_seed[3] = {5,8,1};
unsigned short Nvar3_seed[3] = {9,6,3};


int ET_COMMAND_refine(ET_TCLARGS)
{
  int i, j, value;
  float weight[MAX_INTERVALS], normalizedweight[MAX_INTERVALS];
  float num, sum, sum2, blob, obs, nmean, nstd;
  FILE *inFile, *outFile;

  if( argc < 6 )
    {
      Et_ResultF(interp,
	"Usage: refine subintervals mean std input_file output_file\n");
      return TCL_ERROR;
    }

  value = atoi(argv[1]);
  nmean = atof(argv[2]);
  nstd  = atof(argv[3]);

  if ((inFile = fopen(argv[4],"r")) == NULL)
  {
     Et_ResultF(interp, "Unable to open input file %s.\n",argv[4]);
     return TCL_ERROR;
  }
  outFile = fopen(argv[5],"w");

  while( (i = fscanf(inFile,"%f\n", &num)) != EOF )
    {
      sum = 0.0;
      blob = 1.0 * num;

      for( j = 0; j < value; j++ )
	{
	  /* generate a random weight for each subinterval */
	  weight[j] = (float) Normal(nmean, nstd);

	  /* to avoid negative weightage*/
	  while(weight[j] < 0) 
	  {
	     weight[j] = (float) Normal(nmean, nstd);
	  }
  
	  sum += weight[j];
	}

      sum2 = 0.0;
      /* compute normalized weights */
      for( j = 0; j < value; j++ )
	{
	  normalizedweight[j] = weight[j] / sum;
	  sum2 += normalizedweight[j];
	}

      sum2 = 0.0;
      for( j = 0; j < value; j++ )
	{
	  obs = blob * normalizedweight[j];
	  fprintf(outFile,"%f\n", obs);
	}
    }
  fclose(inFile);
  fclose(outFile);
  return TCL_OK;
}
  
/***********************************************************************/
/*                 RANDOM NUMBER GENERATION STUFF                      */
/***********************************************************************/

/* Parameters for random number generation. */
#define MAX_INT 2147483647       /* Maximum positive integer 2^31 - 1 */

/* the rejection method is used for normal distribution 
   (a) Generate two uniform U(0,1) variates u1 and u2. 
   (b) Let x = -log(u1).
   (c) If u2 > e to the power -(x - 1)square by 2, go back to step
   [a].
   (d) Generate u3.
   (e) If u3 > 0.5, return (mean + (std * x)); otherwise return
   (mean - (std * x)). */

double Normal(float mean, float std)
{
  double var_u1, var_u2, var_u3;
  double x, tmp_val;
  
  var_u1 = erand48(Nvar1_seed);
  var_u2 = erand48(Nvar2_seed);
  x = -log(var_u1);
  tmp_val = exp(-((x - 1)*(x - 1)) / 2);
  while(var_u2 > tmp_val)
    {
      var_u1 = erand48(Nvar1_seed);
      var_u2 = erand48(Nvar2_seed);
      x = -log(var_u1);
      tmp_val = exp(-((x - 1)*(x - 1)) / 2);
    }
  var_u3 = erand48(Nvar3_seed);
  if(var_u3 > 0.5)
    return(mean + (std * x));
  else 
    return(mean - (std * x));
}




