/*F File_name: 
/*F	variance.c
/*
/****************************************************************/
/* This program is to calculates the variances of aggregated    */
/* process X(m). The input of this program will be packet counts*/
/* (a given sample of N observations). We subdivides the whole  */
/* sample into N/m non-overlapping blocks with each has m obser-*/
/* vations and computes the varinaces of x(m).                  */
/****************************************************************/ 

/* Modified by Ram Balakrishnan, March 1999.                  */
//
//N Note: 
//N     This program is altered to be compatiable with cascade.tcl program.
//N     Amalgamate this program with cascade.tcl using "xmktclapp".  

  
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "synTraff.h"

int ET_COMMAND_mkVar(ET_TCLARGS)
{
   FILE *infile, *outfile;
   double mean = 0.0, var, x, first_val;
   int counter = 0, m, i, j, n;
   float *buffer, *p;
   float current;


   if (argc < 3)
   {
      Et_ResultF(interp, "Usage: mkVar infile outfile1\n");
      return TCL_ERROR;
   }

   if ((infile = fopen(argv[1],"r")) == NULL)
   {
      Et_ResultF(interp, "Unable to open input file %s.\n",argv[1]);
      return TCL_ERROR;
   }

   if ((outfile = fopen(argv[2],"w")) == NULL)
   {
      Et_ResultF(interp, "Unable to open output file %s.\n",argv[2]);
      return TCL_ERROR;
   }

   /* Find mean */
   while (fscanf(infile,"%f",&current) != EOF)
   {
      mean += (double) current;
      counter++;
   }

   mean /= counter;


 /*  fprintf(outfile,"Mean of input data : %6.3f\n",mean);*/ 

   if ((buffer = (float *) malloc(counter*sizeof(float))) == NULL)
     {
        Et_ResultF(interp, "Memory allocation error.\n");
        return TCL_ERROR;
     }

    p = buffer;

   rewind(infile);

   while (fscanf(infile,"%f",p++) != EOF);

   m = 1;
   while( m < counter/2 )
   {
    var = 0.0;
    n =  counter/m  ; /* number of non-overlapping intervals */
    for (i = 1; i <= n ; i++)
     { 
        x = 0.0;
        p = buffer+(i-1)*m;
        for (j = 1; j <= m; j++)  /* making a new process X(m) */
           x += (double)(*p++);
        x /= (double) m;        
        var += (x - mean)*(x - mean); 
     }
    var /= n;
    if (m == 1) first_val = var;
    fprintf(outfile ,"%d  %8.6f\n", m, var/first_val);

    if( m < 10 )
	m += 1;
    else if( m < 20 )
	m += 2;
    else if( m < 50 )
	m += 5;
    else if( m < 100 )
	m += 10;
    else if( m < 200 )
	m += 20;
    else if( m < 500 )
	m += 50;
    else if( m < 1000 )
	m += 100;
    else if( m < 2000 )
	m += 200;
    else if( m < 5000 )
	m += 500;
    else if( m < 10000 )
	m += 1000;
    else if( m < 20000 )
	m += 2000;
    else if( m < 50000 )
	m += 5000;
    else if( m < 100000 )
	m += 10000;
    else if( m < 200000 )
	m += 20000;
    else if( m < 500000 )
	m += 50000;
    else m += 100000;
   }

   fclose(infile);
   fclose(outfile);

   return TCL_OK;
}
